
--  -----------------------------------------------------------------------  --
--                               GLOBAL HACKS
--  -----------------------------------------------------------------------  --

--  provide a generic regular expression constructor 
--  based on the most powerful regular expression engine
rex.new = rex.newPCRE
if rex.new == nil then
   rex.new = rex.newPOSIX
end

--  provide additional "util" namespace
util = {}

--  -----------------------------------------------------------------------  --
--                         UNIX grep(1) EMULATIONS
--  -----------------------------------------------------------------------  --

--  "global regular expression print" on file content
function util.grep(expr, filename)
	if not posix.stat(filename, "mode") then
		return nil
	end
	local lines = {}
	local pat = rex.new(expr)
	local pos = 1
	for line in io.lines(filename) do
		if pat:match(line) then
			table.insert(lines, pos, line)
		end
		pos = pos + 1
	end
	if table.getn(lines) == 0 then
		return nil
	end
	return lines
end

--  iterator version
function util.igrep(expr, filename)
	return ipairs(rex.grep(expr, filename))
end

--  boolean version
function util.bgrep(expr, filename)
	if not posix.stat(filename, "mode") then
		return nil
	end
	local pat = rex.new(expr)
	for line in io.lines(filename) do
		if pat:match(line) then
			return true
		end
	end
	return false
end

--  -----------------------------------------------------------------------  --
--                   REGULAR EXPRESSION MATCHING/SUBSTITUTION
--  -----------------------------------------------------------------------  --

--  regular expression based string matching
function util.rmatch(str, regex)
    return rex.new(regex):match(str)
end

--  regular expression based string substitution
function util.rsubst(str, regex, subst)
    local result = str
    local re = rex.new(regex)
    local result_last = ""
    while result ~= result_last do
        result_last = result
        local s, e, m = re:match(result)
        if s ~= nil then
            local prolog = string.sub(result, 1, s - 1)
            local epilog = string.sub(result, e + 1)
            if type(subst) == "function" then
                result = subst(m)
            elseif type(subst) == "table" then
                result = subst[m[1]]
            else
                result = string.gsub(subst, "%$([0-9])", function (n) return m[tonumber(n)] end)
            end
            if result == nil then
                result = ""
            end
            result = prolog .. result .. epilog
        else
            break
        end
    end
    return result
end

--  -----------------------------------------------------------------------  --
--                           RECURSIVE DATA DUMPING
--  -----------------------------------------------------------------------  --

--  WORK HORSE:
--  dump a single object recursively into a string
function util.dump_object(obj, compact)
    local dump = "<unknown>"
    if type(obj) == "nil" then
        dump = "nil"
    elseif type(obj) == "number" then
        dump = string.format("%d", obj)
    elseif type(obj) == "string" then
        local str = obj
        str = string.gsub(str, "\\\\", "\\\\")
        str = string.gsub(str, "\"", "\\\"")
        str = string.gsub(str, "\r", "\\r")
        str = string.gsub(str, "\n", "\\n")
        str = string.gsub(str, ".",
            function(c)
                local n = string.byte(c)
                if n < 32 or n >= 127 then
                    c = string.format("\\%03d", n)
                end
                return c
            end
        )
        dump = "\"" .. str .. "\""
    elseif type(obj) == "boolean" then
        if obj then
            dump = "true"
        else
            dump = "false"
        end
    elseif type(obj) == "table" then
        local space = " "
        if compact ~= nil and compact == true then
            space = ""
        end
        dump = "{"
        local first = true
        for k, v in pairs(obj) do
            if not first then
                dump = dump .. ","
            end
            if string.match(k, "^[a-zA-Z_][a-zA-Z0-9_]*$") == nil then
                k = "[" .. util.dump_object(k, true) .. "]"
            end
            dump = dump .. space .. k .. space .. "=" .. space
            dump = dump .. util.dump_object(v, compact)
            first = false
        end
        dump = dump .. space .. "}"
    elseif type(obj) == "function" then
        dump = "<function>"
    elseif type(obj) == "thread" then
        dump = "<thread>"
    elseif type(obj) == "userdata" then
        dump = "<userdata>"
    end
    return dump
end

--  CONVENIENCE FRONTEND:
--  dump one or more objects recursively, one per line
function util.dump(obj1, ...)
    local dump = util.dump_object(obj1) .. "\n"
    for _, obj in ipairs(arg) do
        dump = dump .. util.dump_object(obj) .. "\n"
    end
    return dump
end

--  CONVENIENCE FRONTEND:
--  dump one or more objects recursively, one per line, to stderr
function util.dump_stderr(obj1, ...)
    io.stderr:write(util.dump(obj1, unpack(arg)))
end

